@@ -9,23 +9,7 @@
 ###############################################################################
 # Copyright (c) 2011, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import sys
@@ -37,6 +21,17 @@ import webserver
 
 from osgeo import gdal, ogr
 
+
+def curl_version():
+    actual_version = [0, 0, 0]
+    for build_info_item in gdal.VersionInfo("BUILD_INFO").strip().split("\n"):
+        if build_info_item.startswith("CURL_VERSION="):
+            actual_version = [
+                int(x) for x in build_info_item[len("CURL_VERSION=") :].split(".")
+            ]
+    return actual_version
+
+
 pytestmark = pytest.mark.require_curl()
 
 ###############################################################################
@@ -222,11 +217,204 @@ def server():
     webserver.server_stop(process, port)
 
 
+###############################################################################
+# Test regular redirection
+
+
+@pytest.mark.parametrize(
+    "authorization_header_allowed", [None, "YES", "NO", "IF_SAME_HOST"]
+)
+def test_vsicurl_test_redirect(server, authorization_header_allowed):
+
+    gdal.VSICurlClearCache()
+
+    expected_headers = None
+    unexpected_headers = []
+    if authorization_header_allowed != "NO":
+        expected_headers = {"Authorization": "Bearer xxx"}
+    else:
+        unexpected_headers = ["Authorization"]
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_redirect/", 404)
+    handler.add(
+        "HEAD",
+        "/test_redirect/test.bin",
+        301,
+        {"Location": "http://localhost:%d/redirected/test.bin" % server.port},
+        expected_headers={"Authorization": "Bearer xxx"},
+    )
+
+    # Curl always forward Authorization if same server when handling itself
+    # the redirect, so this means that CPL_VSIL_CURL_AUTHORIZATION_HEADER_ALLOWED_IF_REDIRECT=NO
+    # is not honored for that particular request. To honour it, we would have
+    # to disable CURLOPT_FOLLOWLOCATION and implement it at hand
+    handler.add(
+        "HEAD",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        expected_headers={"Authorization": "Bearer xxx"},
+    )
+
+    handler.add(
+        "GET",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        b"xyz",
+        expected_headers=expected_headers,
+        unexpected_headers=unexpected_headers,
+    )
+
+    options = {"GDAL_HTTP_HEADERS": "Authorization: Bearer xxx"}
+    if authorization_header_allowed:
+        options[
+            "CPL_VSIL_CURL_AUTHORIZATION_HEADER_ALLOWED_IF_REDIRECT"
+        ] = authorization_header_allowed
+    with webserver.install_http_handler(handler), gdal.config_options(options):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test_redirect/test.bin" % server.port,
+            "rb",
+        )
+        assert f is not None
+        try:
+            assert gdal.VSIFReadL(1, 3, f) == b"xyz"
+        finally:
+            gdal.VSIFCloseL(f)
+
+
+###############################################################################
+# Test regular redirection
+
+
+@pytest.mark.parametrize(
+    "authorization_header_allowed", [None, "YES", "NO", "IF_SAME_HOST"]
+)
+def test_vsicurl_test_redirect_different_server(server, authorization_header_allowed):
+
+    gdal.VSICurlClearCache()
+
+    expected_headers = None
+    unexpected_headers = []
+    if authorization_header_allowed == "YES":
+        expected_headers = {"Authorization": "Bearer xxx"}
+    else:
+        unexpected_headers = ["Authorization"]
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_redirect/", 404)
+    handler.add(
+        "HEAD",
+        "/test_redirect/test.bin",
+        301,
+        {"Location": "http://127.0.0.1:%d/redirected/test.bin" % server.port},
+        expected_headers={"Authorization": "Bearer xxx"},
+    )
+    handler.add(
+        "HEAD",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        expected_headers=expected_headers,
+        unexpected_headers=unexpected_headers,
+    )
+    handler.add(
+        "GET",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        b"xyz",
+        expected_headers=expected_headers,
+        unexpected_headers=unexpected_headers,
+    )
+
+    options = {"GDAL_HTTP_HEADERS": "Authorization: Bearer xxx"}
+    if authorization_header_allowed:
+        options[
+            "CPL_VSIL_CURL_AUTHORIZATION_HEADER_ALLOWED_IF_REDIRECT"
+        ] = authorization_header_allowed
+    with webserver.install_http_handler(handler), gdal.config_options(options):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test_redirect/test.bin" % server.port,
+            "rb",
+        )
+        try:
+            assert gdal.VSIFReadL(1, 3, f) == b"xyz"
+        finally:
+            gdal.VSIFCloseL(f)
+
+
+###############################################################################
+# Test regular redirection
+
+
+@gdaltest.enable_exceptions()
+@pytest.mark.require_curl(7, 61, 0)
+@pytest.mark.parametrize(
+    "authorization_header_allowed", [None, "YES", "NO", "IF_SAME_HOST"]
+)
+def test_vsicurl_test_redirect_different_server_with_bearer(
+    server, authorization_header_allowed
+):
+
+    gdal.VSICurlClearCache()
+
+    expected_headers = None
+    unexpected_headers = []
+    if authorization_header_allowed == "YES":
+        expected_headers = {"Authorization": "Bearer xxx"}
+    else:
+        unexpected_headers = ["Authorization"]
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_redirect/", 404)
+    handler.add(
+        "HEAD",
+        "/test_redirect/test.bin",
+        301,
+        {"Location": "http://127.0.0.1:%d/redirected/test.bin" % server.port},
+        expected_headers={"Authorization": "Bearer xxx"},
+    )
+    handler.add(
+        "HEAD",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        expected_headers=expected_headers,
+        unexpected_headers=unexpected_headers,
+    )
+    handler.add(
+        "GET",
+        "/redirected/test.bin",
+        200,
+        {"Content-Length": "3"},
+        b"xyz",
+        expected_headers=expected_headers,
+        unexpected_headers=unexpected_headers,
+    )
+
+    options = {"GDAL_HTTP_AUTH": "BEARER", "GDAL_HTTP_BEARER": "xxx"}
+    if authorization_header_allowed:
+        options[
+            "CPL_VSIL_CURL_AUTHORIZATION_HEADER_ALLOWED_IF_REDIRECT"
+        ] = authorization_header_allowed
+    with webserver.install_http_handler(handler), gdal.config_options(options):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test_redirect/test.bin" % server.port,
+            "rb",
+        )
+        try:
+            assert gdal.VSIFReadL(1, 3, f) == b"xyz"
+        finally:
+            gdal.VSIFCloseL(f)
+
+
 ###############################################################################
 # Test redirection with Expires= type of signed URLs
 
 
-def test_vsicurl_test_redirect(server):
+def test_vsicurl_test_redirect_with_expires(server):
 
     gdal.VSICurlClearCache()
 
@@ -582,9 +770,16 @@ def test_vsicurl_test_retry(server):
         )
         data_len = 0
         if f:
-            data_len = len(gdal.VSIFReadL(1, 1, f))
-            gdal.VSIFCloseL(f)
-        assert data_len == 0
+            try:
+                data_len = len(gdal.VSIFReadL(1, 1, f))
+                assert data_len == 0
+                assert gdal.VSIFEofL(f) == 0
+                assert gdal.VSIFErrorL(f) == 1
+                gdal.VSIFClearErrL(f)
+                assert gdal.VSIFEofL(f) == 0
+                assert gdal.VSIFErrorL(f) == 0
+            finally:
+                gdal.VSIFCloseL(f)
 
     gdal.VSICurlClearCache()
 
@@ -601,13 +796,108 @@ def test_vsicurl_test_retry(server):
             "rb",
         )
         assert f is not None
+        try:
+            gdal.ErrorReset()
+            with gdal.quiet_errors():
+                data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+                assert data == "foo"
+            error_msg = gdal.GetLastErrorMsg()
+            assert "429" in error_msg
+            assert gdal.VSIFEofL(f) == 0
+            assert gdal.VSIFErrorL(f) == 0
+        finally:
+            gdal.VSIFCloseL(f)
+
+
+###############################################################################
+
+
+def test_vsicurl_retry_codes_ALL(server):
+
+    gdal.VSICurlClearCache()
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_retry/", 404)
+    handler.add("HEAD", "/test_retry/test.txt", 200, {"Content-Length": "3"})
+    handler.add(
+        "GET", "/test_retry/test.txt", 400
+    )  #  non retriable by default, but here allowed because of retry_codes=ALL
+    handler.add("GET", "/test_retry/test.txt", 200, {}, "foo")
+    with webserver.install_http_handler(handler):
+        f = gdal.VSIFOpenL(
+            "/vsicurl?max_retry=1&retry_delay=0.01&retry_codes=ALL&url=http://localhost:%d/test_retry/test.txt"
+            % server.port,
+            "rb",
+        )
+        assert f is not None
         gdal.ErrorReset()
         with gdal.quiet_errors():
             data = gdal.VSIFReadL(1, 3, f).decode("ascii")
         error_msg = gdal.GetLastErrorMsg()
         gdal.VSIFCloseL(f)
         assert data == "foo"
-        assert "429" in error_msg
+        assert "400" in error_msg
+
+
+###############################################################################
+
+
+def test_vsicurl_retry_codes_enumerated(server):
+
+    gdal.VSICurlClearCache()
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_retry/", 404)
+    handler.add("HEAD", "/test_retry/test.txt", 200, {"Content-Length": "3"})
+    handler.add(
+        "GET", "/test_retry/test.txt", 400
+    )  #  non retriable by default, but here allowed because of retry_codes=ALL
+    handler.add("GET", "/test_retry/test.txt", 200, {}, "foo")
+    with webserver.install_http_handler(handler), gdal.config_option(
+        "GDAL_HTTP_RETRY_CODES", "400"
+    ):
+        f = gdal.VSIFOpenL(
+            "/vsicurl?max_retry=1&retry_delay=0.01&url=http://localhost:%d/test_retry/test.txt"
+            % server.port,
+            "rb",
+        )
+        assert f is not None
+        gdal.ErrorReset()
+        with gdal.quiet_errors():
+            data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+        error_msg = gdal.GetLastErrorMsg()
+        gdal.VSIFCloseL(f)
+        assert data == "foo"
+        assert "400" in error_msg
+
+
+###############################################################################
+
+
+def test_vsicurl_retry_codes_no_match(server):
+
+    gdal.VSICurlClearCache()
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/test_retry/", 404)
+    handler.add("HEAD", "/test_retry/test.txt", 200, {"Content-Length": "3"})
+    handler.add(
+        "GET", "/test_retry/test.txt", 400
+    )  #  non retriable by default, and not listed in GDAL_HTTP_RETRY_CODES
+    with webserver.install_http_handler(handler), gdal.config_option(
+        "GDAL_HTTP_RETRY_CODES", "409"
+    ):
+        f = gdal.VSIFOpenL(
+            "/vsicurl?max_retry=1&retry_delay=0.01&url=http://localhost:%d/test_retry/test.txt"
+            % server.port,
+            "rb",
+        )
+        assert f is not None
+        gdal.ErrorReset()
+        with gdal.quiet_errors():
+            data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+        gdal.VSIFCloseL(f)
+        assert len(data) == 0
 
 
 ###############################################################################
@@ -1026,21 +1316,49 @@ def test_vsicurl_GDAL_HTTP_HEADERS(server):
     filename = (
         "/vsicurl/http://localhost:%d/test_vsicurl_GDAL_HTTP_HEADERS.bin" % server.port
     )
-    gdal.SetPathSpecificOption(
-        filename,
-        "GDAL_HTTP_HEADERS",
-        r'Foo: Bar,"Baz: escaped backslash \\, escaped double-quote \", end of value",Another: Header',
-    )
-    with webserver.install_http_handler(handler):
-        statres = gdal.VSIStatL(filename)
-    gdal.SetPathSpecificOption(filename, "GDAL_HTTP_HEADERS", None)
-    assert statres.size == 3
+    try:
+        gdal.SetPathSpecificOption(
+            filename,
+            "GDAL_HTTP_HEADERS",
+            r'Foo: Bar,"Baz: escaped backslash \\, escaped double-quote \", end of value",Another: Header',
+        )
+        with webserver.install_http_handler(handler):
+            statres = gdal.VSIStatL(filename)
+        assert statres.size == 3
+
+        gdal.VSICurlClearCache()
+        handler = webserver.SequentialHandler()
+        handler.add(
+            "HEAD",
+            "/test_vsicurl_GDAL_HTTP_HEADERS.bin",
+            200,
+            {"Content-Length": "3"},
+            expected_headers={
+                "Foo": "Bar",
+                "Baz": r'escaped backslash \, escaped double-quote ", end of value',
+                "Another": "Header",
+            },
+        )
+        with webserver.install_http_handler(handler):
+            statres = gdal.VSIStatL(
+                "/vsicurl_streaming/http://localhost:%d/test_vsicurl_GDAL_HTTP_HEADERS.bin"
+                % server.port
+            )
+        assert statres.size == 3
+
+    finally:
+        gdal.SetPathSpecificOption(filename, "GDAL_HTTP_HEADERS", None)
 
 
 ###############################################################################
 # Test CPL_VSIL_CURL_USE_HEAD=NO
 
 
+# Cf fix of https://github.com/curl/curl/pull/14390
+@pytest.mark.skipif(
+    curl_version() == [8, 9, 1],
+    reason="fail with SIGPIPE with curl 8.9.1",
+)
 def test_vsicurl_test_CPL_VSIL_CURL_USE_HEAD_NO(server):
 
     gdal.VSICurlClearCache()
@@ -1170,3 +1488,89 @@ def test_vsicurl_404_repeated_same_resource(server):
 
     with pytest.raises(Exception, match="404"):
         gdal.Open("/vsicurl/http://localhost:%d/does/not/exist.bin" % server.port)
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+def test_vsicurl_cache_control_not_set(server):
+
+    gdal.VSICurlClearCache()
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/", 404)
+    handler.add("HEAD", "/test.txt", 200, {"Content-Length": "3"})
+    handler.add("GET", "/test.txt", 200, {}, "foo")
+    with webserver.install_http_handler(handler):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test.txt" % server.port,
+            "rb",
+        )
+        assert f is not None
+        data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+        gdal.VSIFCloseL(f)
+        assert data == "foo"
+
+    with webserver.install_http_handler(webserver.SequentialHandler()):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test.txt" % server.port,
+            "rb",
+        )
+        assert f is not None
+        data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+        gdal.VSIFCloseL(f)
+        assert data == "foo"
+
+
+###############################################################################
+
+
+@gdaltest.enable_exceptions()
+def test_vsicurl_cache_control_no_cache(server):
+
+    gdal.VSICurlClearCache()
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/", 404)
+    handler.add(
+        "HEAD", "/test.txt", 200, {"Content-Length": "3", "Cache-Control": "no-cache"}
+    )
+    handler.add(
+        "GET",
+        "/test.txt",
+        200,
+        {"Content-Length": "3", "Cache-Control": "no-cache"},
+        "foo",
+    )
+    with webserver.install_http_handler(handler):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test.txt" % server.port,
+            "rb",
+        )
+        assert f is not None
+        data = gdal.VSIFReadL(1, 3, f).decode("ascii")
+        gdal.VSIFCloseL(f)
+        assert data == "foo"
+
+    handler = webserver.SequentialHandler()
+    handler.add("GET", "/", 404)
+    handler.add(
+        "HEAD", "/test.txt", 200, {"Content-Length": "6", "Cache-Control": "no-cache"}
+    )
+    handler.add(
+        "GET",
+        "/test.txt",
+        200,
+        {"Content-Length": "6", "Cache-Control": "no-cache"},
+        "barbaz",
+    )
+    with webserver.install_http_handler(handler):
+        f = gdal.VSIFOpenL(
+            "/vsicurl/http://localhost:%d/test.txt" % server.port,
+            "rb",
+        )
+        assert f is not None
+        data = gdal.VSIFReadL(1, 6, f).decode("ascii")
+        gdal.VSIFCloseL(f)
+        assert data == "barbaz"
