@@ -9,23 +9,7 @@
 ###############################################################################
 # Copyright (c) 2018, Even Rouault <even dot rouault at spatialys dot com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import json
@@ -1237,7 +1221,9 @@ def test_ogr_mvt_write_mbtiles():
 
 @pytest.mark.require_driver("SQLite")
 @pytest.mark.require_geos
-def test_ogr_mvt_write_limitations_max_size():
+@pytest.mark.parametrize("implicit_limitation", [True, False])
+@gdaltest.enable_exceptions()
+def test_ogr_mvt_write_limitations_max_size(implicit_limitation, tmp_path):
 
     src_ds = gdal.GetDriverByName("Memory").Create("", 0, 0, 0, gdal.GDT_Unknown)
     lyr = src_ds.CreateLayer("mylayer")
@@ -1261,10 +1247,22 @@ def test_ogr_mvt_write_limitations_max_size():
         out_ds = gdal.VectorTranslate(
             "/vsimem/out.mbtiles",
             src_ds,
-            datasetCreationOptions=["MAX_SIZE=100", "SIMPLIFICATION=1"],
+            datasetCreationOptions=[
+                "@MAX_SIZE_FOR_TEST=101" if implicit_limitation else "MAX_SIZE=101",
+                "SIMPLIFICATION=1",
+            ],
         )
     assert out_ds is not None
-    out_ds = None
+    gdal.ErrorReset()
+    with gdal.quiet_errors():
+        out_ds.Close()
+    if implicit_limitation:
+        assert (
+            gdal.GetLastErrorMsg()
+            == "At least one tile exceeded the default maximum tile size of 101 bytes and was encoded at lower resolution"
+        )
+    else:
+        assert gdal.GetLastErrorMsg() == ""
 
     out_ds = ogr.Open("/vsimem/out.mbtiles")
     assert out_ds is not None
@@ -1458,7 +1456,9 @@ def test_ogr_mvt_write_limitations_max_size_polygon():
 
 @pytest.mark.require_driver("SQLite")
 @pytest.mark.require_geos
-def test_ogr_mvt_write_limitations_max_features():
+@pytest.mark.parametrize("implicit_limitation", [True, False])
+@gdaltest.enable_exceptions()
+def test_ogr_mvt_write_limitations_max_features(implicit_limitation):
 
     src_ds = gdal.GetDriverByName("Memory").Create("", 0, 0, 0, gdal.GDT_Unknown)
     lyr = src_ds.CreateLayer("mylayer")
@@ -1479,10 +1479,21 @@ def test_ogr_mvt_write_limitations_max_features():
         "/vsimem/out.mbtiles",
         src_ds,
         format="MVT",
-        datasetCreationOptions=["MAX_FEATURES=1"],
+        datasetCreationOptions=[
+            "@MAX_FEATURES_FOR_TEST=1" if implicit_limitation else "MAX_FEATURES=1"
+        ],
     )
     assert out_ds is not None
-    out_ds = None
+    gdal.ErrorReset()
+    with gdal.quiet_errors():
+        out_ds.Close()
+    if implicit_limitation:
+        assert (
+            gdal.GetLastErrorMsg()
+            == "At least one tile exceeded the default maximum number of features per tile (1) and was truncated to satisfy it."
+        )
+    else:
+        assert gdal.GetLastErrorMsg() == ""
 
     out_ds = ogr.Open("/vsimem/out.mbtiles")
     assert out_ds is not None
